//+------------------------------------------------------------------+
//|                                          Jarroo Level Picker.mq4 |
//|      Copyright @2013, Eric Schroeder, Wizardblue, MOpoka | Jackt |
//|                                                                  |
//+------------------------------------------------------------------+


// V1.0 First version by Eric Schroeder

// V1.1 includes description for Hline by Wizardblue

// V1.2 code cleaned up by MOpoka

// V1.3 add visibility option requested by Jaz

// V1.4 by MOpoka: descriptions are separated by SPACEs depending on period

// V1.5 by Eric: modified descriptions not to overlap

// V1.5a by Jackt: Two lines instead of one

// V1.6 by MOpoka: Whether LCS or HCR is generated now depends on distance between place when script was dropped and Open\Close + better customization and support for H1 and H4 timeframes.

// V1.7 Limiting visibility is now optional as well as displaying level type (HCR or LCS) in descripiton.
//PARAMETERS
bool  UseTwoLines       = false;   // Set it to false in order to draw only one thick horizontal line as in pre 1.5a version
bool  DescOnLeft        = true;   // Do you want description (like D HCR) to be drawn on the left side of the chart?
bool  DescOnSwing       = false;   // Do you want description (like D HCR) to be drawn near swing point?
bool  DisplayLevelType  = true;

color HourlyColor       = PaleGreen;
color H4Color           = Turquoise;
color DailyColor        = Green;
color WeeklyColor       = OrangeRed;
color MonthlyColor      = MediumVioletRed;

bool  DrawAsBackground  = true;
int   LineStyle         = STYLE_SOLID;
int   LineWidth         = 2;
bool  LimitVisibility   = true;

//Constants
#define SECONDS_IN_MINUTE 60
#define SPACE "            "
   
int start()
{
   color lineColor;
   string periodLetter;
   string description;

   int swingPoint = iBarShift(Symbol(),0,WindowTimeOnDropped(),false);

   double candleBodyHigh = Close[swingPoint];
   double candleBodyLow = Open[swingPoint];
   
   bool isBullishBar = true;
   int candlePart;
      
   if(Close[swingPoint] < Open[swingPoint])
   {
      candleBodyHigh = Open[swingPoint];
      candleBodyLow = Close[swingPoint];
      isBullishBar = false;
   }

   datetime trendlineStartTime = Time[swingPoint];
   datetime trendlineEndTime = Time[swingPoint-2];
   
   double price = candleBodyHigh;
   string suffix = "HCR";   
   
   double distanceToCandleBodyHigh = MathAbs(WindowPriceOnDropped() - candleBodyHigh);
   double distanceToCandleBodyLow  = MathAbs(WindowPriceOnDropped() - candleBodyLow);
  
   if(distanceToCandleBodyLow < distanceToCandleBodyHigh)
   {
      price = candleBodyLow;
      suffix = "LCS";
    
      if(isBullishBar)
         candlePart = MODE_OPEN;
      else 
         candlePart = MODE_CLOSE;
   }
   else
   {
      if(isBullishBar)
         candlePart = MODE_CLOSE;
      else 
         candlePart = MODE_OPEN;
   }
   
   if(!DisplayLevelType)   
      suffix = "";
      
   switch(Period()) 
   {
         case PERIOD_H1: 
            lineColor = HourlyColor;
            periodLetter = "H1";
            description  = SPACE+SPACE+SPACE+SPACE+periodLetter+" "+suffix;
            if(candlePart == MODE_CLOSE)
            {
               trendlineStartTime = trendlineStartTime + (PERIOD_H1 * SECONDS_IN_MINUTE)-SECONDS_IN_MINUTE; // shift to bar close time
            }
            break;
         case PERIOD_H4: 
            lineColor = H4Color;
            periodLetter = "H4";
            description  = SPACE+SPACE+SPACE+periodLetter+" "+suffix;
            if(candlePart == MODE_CLOSE)
            {
               trendlineStartTime = trendlineStartTime + (PERIOD_H4 * SECONDS_IN_MINUTE)-SECONDS_IN_MINUTE; // shift to bar close time
            }
            break;
         case PERIOD_D1: 
            lineColor = DailyColor;
            periodLetter = "D";
            description  = SPACE+SPACE+periodLetter+" "+suffix;
            if(candlePart == MODE_CLOSE)
            {
               trendlineStartTime = trendlineStartTime + (PERIOD_D1 * SECONDS_IN_MINUTE)-SECONDS_IN_MINUTE; // shift to bar close time
            }
            break;
         case PERIOD_W1: 
            lineColor = WeeklyColor;
            periodLetter = "W";
            description  = SPACE+periodLetter+" "+suffix;
            
            if(candlePart == MODE_OPEN)
               trendlineStartTime = trendlineStartTime + (PERIOD_D1 * SECONDS_IN_MINUTE); //FxPro uses Sunday 0:00 for weekly candles; Shift to Monday 0:00
            else 
               trendlineStartTime = trendlineStartTime + 6 * (PERIOD_D1 * SECONDS_IN_MINUTE) - SECONDS_IN_MINUTE; // shift to Friday 23:59 (New York 16:59)
            break;
         case PERIOD_MN1: 
            lineColor = MonthlyColor;
            periodLetter = "M";
            description  = periodLetter+" "+suffix;
            break;
   }
   
   
   if(UseTwoLines)
   {
      DrawTrendline(trendlineStartTime,trendlineEndTime,price,lineColor,description);
      DrawHorizontalLine(price,lineColor,description);
   }
   else
   {
      DrawHorizontalLine(price,lineColor,description);
   }
}


void DrawTrendline(datetime trendlineStartTime, datetime trendlineEndTime, double price,color lineColor, string description)
{
    int period = Period();
    string lineName = "HCRLCS_Manual_TL_"+Period()+"@"+DoubleToStr(price,Digits);
    
    if(ObjectFind(lineName)>=0)
      ObjectDelete(lineName);
   
    ObjectCreate(lineName, OBJ_TREND, 0, trendlineStartTime, price, trendlineEndTime, price);
    ObjectSet(lineName, OBJPROP_STYLE, LineStyle);
    ObjectSet(lineName, OBJPROP_WIDTH, LineWidth);
    ObjectSet(lineName, OBJPROP_BACK, DrawAsBackground);
    ObjectSet(lineName, OBJPROP_COLOR, lineColor);
    ObjectSet(lineName, OBJPROP_RAY, true);
    
    if(DescOnSwing)
       ObjectSetText(lineName, description);
    if(LimitVisibility)
      ApplyPeriodVisibility(period,lineName);
}

void DrawHorizontalLine(double price,color lineColor, string description)
{
    int period = Period();
    string lineName = "HCRLCS_Manual_HL_"+Period()+"@"+DoubleToStr(price,Digits);
    if(ObjectFind(lineName)!=0)
    {
      ObjectCreate(lineName, OBJ_HLINE, 0, NULL, price);
      if(UseTwoLines)
      {
         ObjectSet(lineName, OBJPROP_STYLE, STYLE_DOT);
         ObjectSet(lineName, OBJPROP_WIDTH, 1);
      }
      else
      {
         ObjectSet(lineName, OBJPROP_STYLE, LineStyle);
         ObjectSet(lineName, OBJPROP_WIDTH, LineWidth);
      }
      ObjectSet(lineName, OBJPROP_BACK, DrawAsBackground);
      ObjectSet(lineName, OBJPROP_COLOR, lineColor);
      if(DescOnLeft)
         ObjectSetText(lineName, description);
      if(LimitVisibility)
         ApplyPeriodVisibility(period,lineName);
    }
}

//Make object visible only on specified period and below
void ApplyPeriodVisibility(int period,string objectName)
{
   int visibility=OBJ_PERIOD_M1;
   
   if(period>=PERIOD_M5)
   visibility|=OBJ_PERIOD_M5;
   
   if(period>=PERIOD_M15)
   visibility|=OBJ_PERIOD_M15;
   
   if(period>=PERIOD_M30)
   visibility|=OBJ_PERIOD_M30;
   
   if(period>=PERIOD_H1)
   visibility|=OBJ_PERIOD_H1;
   
   if(period>=PERIOD_H4)
   visibility|=OBJ_PERIOD_H4;
   
   if(period>=PERIOD_D1)
   visibility|=OBJ_PERIOD_D1;
   
   if(period>=PERIOD_W1)
   visibility|=OBJ_PERIOD_W1;
   
   if(period>=PERIOD_MN1)
   visibility|=OBJ_PERIOD_MN1;
   
   ObjectSet(objectName,OBJPROP_TIMEFRAMES,visibility);
}